# -*- coding: utf8 -*-
from __future__ import unicode_literals

import datetime

parser_class = "biosensor"


def test_decode_qualitative_result(self):
    date = datetime.datetime(2018, 8, 30, 17, 17, 1)
    worklist = self.decode(
        """
    MSH|^~\\&|FA20B02VA0234^0000000000000000^EUI-64|sdb|||20180831104332-0500||ORU^R01^ORU_R01|{519a5093-a927-4eff-b906-1468fea313a6}|P|2.6|||AL|NE||UNICODE UTF8|EN^English^ISO639||IHE_PCD_ORU_R01^IHE PCD^1.3.6.1.4.1.19376.1.6.4.1^ISO
    PID|||||^^^^^^U
    OBR|1|db524aea-6ed8-4d12-b08d-cf441dd73fb7^FA20B02VA0234^0000000000000000^GUID|db524aea6ed8-4d12-b08d-cf441dd73fb7^FA20B02VA0234^0000000000000000^GUID|6557-3^Group A Streptococcus^LN|||20180830171701-0500|20180830171701-0500
    NTE|1||Device Information,Date of manufacture=20180528,LotNo=141,SerialNo=00000,Kind=FLine23
    OBX|1|ST|6557-3^Group A Streptococcus^LN|1.0.0.3|Negative|||N|||F|||20180830171701-0500||guest|||20180830171701-0500
    NTE|1||Cut Off Index,Value=0.15
    """
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "results": [
                    {
                        "code": "Group A Streptococcus",
                        "date": date,
                        "operator": "guest",
                        "value": "Negative",
                        "flags": ["N", "F"],
                        "lots": [{"lot": "141", "type": "reagent"}],
                    }
                ],
                "type": "patient",
                "sample_id": "",
            }
        ],
    }
    assert worklist == expected


def test_decode_cut_off_index_result(self):
    date = datetime.datetime(2021, 9, 14, 10, 42, 36)
    worklist = self.decode(
        """
    MSH|^~\\&|FA24B02AA0061^70b3d573725002a3^EUI-64| |||20210914113249+0100||ORU^R01^ORU_R01|{ca36b6eb-215e-49bc-bf9c-0d1d489c9f3d}|P|2.6|||AL|NE||UNICODE UTF-8|EN^English^ISO639||IHE_PCD_ORU_R01^IHE PCD^1.3.6.1.4.1.19376.1.6.4.1^ISO
    PID|||394509141043||^^^^^^U
    OBR|1|047e9487-c88c-43ec-b3c1-302a6b0abf1f^FA24B02AA0061^70b3d573725002a3^GUID|047e9487-c88c-43ec-b3c1-302a6b0abf1f^FA24B02AA0061^70b3d573725002a3^GUID|6553-2^Streptococcus pneumoniae ^LN|||20210914104236+0100|20210914104236+0100
    NTE|1||Device Information,Date of manufacture=20210219,LotNo=011,SerialNo=72327,Kind=FLine23
    OBX|1||6553-2^Streptococcus pneumoniae^LN|1.0.0.5||||N|||F|||20210914104236+0100||admin|||20210914104236+0100
    NTE|1||Cut Off Index,Value=0.17
    """
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "results": [
                    {
                        "code": "Streptococcus pneumoniae Flag",
                        "date": date,
                        "operator": "admin",
                        "value": "N",
                    },
                    {
                        "code": "Streptococcus pneumoniae Cut Off Index",
                        "date": date,
                        "flags": ["N", "F"],
                        "lots": [{"lot": "011", "type": "reagent"}],
                        "operator": "admin",
                        "value": "0.17",
                    },
                ],
                "sample_id": "394509141043",
                "type": "patient",
            }
        ],
    }
    assert worklist == expected


def test_decode_quantitative_result(self):
    date = datetime.datetime(2017, 1, 30, 14, 48, 34)
    worklist = self.decode(
        """
    MSH|^~\\&|FA20A01XA0026^70b3d57372300741^EUI-64||||20170201141842-0500||ORU^R01^ORU_R01|{c0e4c073-0829-4716-89a8-c815747989cb}|P|2.6|||AL|NE||UNICODE UTF8|EN^English^ISO639||IHE_PCD_ORU_R01^IHE PCD^1.3.6.1.4.1.19376.1.6.4.1^ISO
    PID|||||^^^^^^U
    OBR|1|7ea0b17e-bf40-40e1-9478-7ba78ccfb7a9^FA20A01XA0026^70b3d57372300741^GUID|7ea0b17ebf40-40e1-9478-7ba78ccfb7a9^FA20A01XA0026^70b3d57372300741^GUID|55454-3^Hemoglobin A1c^LN|||20170130144834-0500|20170130144834-0500
    NTE|1||Device Information,Date of manufacture=20160811,LotNo=010,SerialNo=00000,Kind=FLine1
    OBX|1|NM|55454-3^Hemoglobin A1c^LN|1.0.0.7|9.91|%^Percent^NGSP|[4.0;15.0]||||F|||20170130144834-0500||guest|||20170130144834-0500
    """
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "results": [
                    {
                        "code": "Hemoglobin A1c",
                        "date": date,
                        "operator": "guest",
                        "unit": "%",
                        "value": "9.91",
                        "flags": ["F"],
                        "lots": [{"lot": "010", "type": "reagent"}],
                    }
                ],
                "type": "patient",
                "sample_id": "",
            }
        ],
    }
    assert worklist == expected


def test_decode_cwe_result(self):
    date = datetime.datetime(2017, 12, 4, 12, 16, 1)
    worklist = self.decode(
        """
    MSH|^~\\&|FA20B01AA0265^0000000000000000^EUI-64||||20180123100803-0500||ORU^R01^ORU_R01|{d4acc100-7cdd-45dd-bf26-83045c48fb0d}|P|2.6|||AL|NE||UNICODE UTF8|EN^English^ISO639||IHE_PCD_ORU_R01^IHE PCD^1.3.6.1.4.1.19376.1.6.4.1^ISO
    PID|||151089||^^^^^^U
    OBR|1|3a06b5b0-aad6-41ed-971d-75a101613ab0^FA20B01AA0265^0000000000000000^GUID|3a06b5b0-aad6-41ed-971d-75a101613ab0^FA20B01AA0265^0000000000000000^GUID|72365-0^Influenza A/B^LN|||20171204121601-0500|20171204121601-0500
    NTE|1||Device Information,Date of manufacture=20170703,LotNo=015,SerialNo=10502,Kind=FLine23
    OBX|1|CWE|72365-0^Influenza A/B^LN|1.0.0.2|LA19017-5^Influenza A virus positive^LN||||||F|||20171204121601-0500||guest|||20171204121601-0500
    NTE|1||Cut Off Index,Value=238.74
    OBX|2|CWE|72365-0^Influenza A/B^LN|1.0.0.2|LA19020-9^Influenza B virus negative^LN||||||F|||20171204121601-0500||guest|||20171204121601-0500
    NTE|2||Cut Off Index,Value=0.04
    """
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "results": [
                    {
                        "code": "Influenza A/B LA19017-5",
                        "date": date,
                        "operator": "guest",
                        "value": "Influenza A virus positive",
                        "flags": ["F"],
                        "lots": [{"lot": "015", "type": "reagent"}],
                    },
                    {
                        "code": "Influenza A/B LA19020-9",
                        "date": date,
                        "operator": "guest",
                        "value": "Influenza B virus negative",
                        "flags": ["F"],
                        "lots": [{"lot": "015", "type": "reagent"}],
                    },
                ],
                "type": "patient",
                "sample_id": "151089",
            }
        ],
    }
    assert worklist == expected


def test_decode_qualitative_control(self):
    date = datetime.datetime(2018, 3, 15, 15, 28, 43)
    worklist = self.decode(
        """
    MSH|^~\\&|FA20B02VA0234^0000000000000000^EUI-64||||20180321105734-0500||ORU^R01^ORU_R01|{331bc9d1-032f-4ad6-8f1c-79637d7e229f}|P|2.6|||AL|NE||UNICODE UTF8|EN^English^ISO639||IHE_PCD_ORU_R01^IHE PCD^1.3.6.1.4.1.19376.1.6.4.1^ISO
    OBR|1|e8042043-6235-468e-95d2-e28f49fc21cd^FA20B02VA0234^0000000000000000^GUID|e8042043-6235-468e-95d2-e28f49fc21cd^FA20B02VA0234^0000000000000000^GUID|72365-0^Influenza A/B^LN|||20180315152843-0500|20180315152843-0500
    NTE|1||Device Information,Date of manufacture=20170822,LotNo=013,SerialNo=12467,Kind=FLine23
    OBX|1|ST|QC-2^Influenza A Control Set Negative^SDB|1.0.0.2|Negative|||Pass|||F|||20180315152843-0500||guest|||20180315152843-0500
    NTE|1||Cut Off Index,Value=0.00
    OBX|2|ST|QC-2^Influenza B Control Set Negative^SDB|1.0.0.2|Negative|||Pass|||F|||20180315152843-0500||guest|||20180315152843-0500
    NTE|2||Cut Off Index,Value=0.11
    """
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "results": [
                    {
                        "code": "Influenza A Control Set Negative",
                        "date": date,
                        "operator": "guest",
                        "value": "Negative",
                        "flags": ["Pass", "F"],
                    },
                    {
                        "code": "Influenza B Control Set Negative",
                        "date": date,
                        "operator": "guest",
                        "value": "Negative",
                        "flags": ["Pass", "F"],
                    },
                ],
                "type": "control",
                "control_lot": "013",
                "sample_id": "QC-2",
            }
        ],
    }
    assert worklist == expected


def test_decode_quantitative_control(self):
    date = datetime.datetime(2017, 5, 12, 9, 1, 45)
    worklist = self.decode(
        """
    MSH|^~\\&|FA20A03AA0155^70b3d57372300be7^EUI-64||||20170201142811-0500||ORU^R01^ORU_R01|{b65ba417-2f2f-4390-838f-9710b291dc8a}|P|2.6|||AL|NE||UNICODE UTF8|EN^English^ISO639||IHE_PCD_ORU_R01^IHE PCD^1.3.6.1.4.1.19376.1.6.4.1^ISO
    OBR|1|9a3b0abe-5d4d-4154-a3a1-92fb0aa88829^FA20A03AA0155^70b3d57372300be7^GUID|9a3b0abe5d4d-4154-a3a1-92fb0aa88829^FA20A03AA0155^70b3d57372300be7^GUID|14957-5^Microalbumin^LN|||20170512090145-0500|20170512090145-0500
    NTE|1||Device Information,Date of manufacture=20161215,LotNo=011,SerialNo=00000,Kind=FLine1
    OBX|1|NM|QC-8^Microalbumin^SDB|1.0.0.8|145.8|mg/L^milligram per liter^UCUM|[123.0;222.0]|Pass|||F|||20170512090145-0500||01|||20170512090145-0500
    """
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "results": [
                    {
                        "code": "Microalbumin",
                        "date": date,
                        "operator": "01",
                        "unit": "mg/L",
                        "value": "145.8",
                        "flags": ["Pass", "F"],
                    }
                ],
                "type": "control",
                "control_lot": "011",
                "sample_id": "QC-8",
            }
        ],
    }
    assert worklist == expected
