# -*- coding: utf8 -*-
from __future__ import unicode_literals

import copy
import datetime

import driverng

parser_class = "dms"


def test_encode_order(self):
    message = self.order()
    expected = """
    H|\\^&||||||||||P|1|20130715141912
    P|1|036666666|L036666666||NYME^ANO|NJF|20110513|F|||||PRESCRIBER|||1.70^cm|58^kg||||||20130511103030||SER^SERVICE
    O|1|123456||^^^A^^^1:10|S||20130513114242||||N||||Serum||||||||||O
    O|2|123456||^^^B^^RR|S||20130513114242||||A||||Serum||||||||||O
    O|3|123456||^^^C|S||20130513114242||||C||||Serum||||||||||O
    L|1
    """
    assert message == expected


def test_encode_order_with_options(self):
    self.configure(
        dms_envoyer_no_demandeur=True,
        dms_envoyer_no_patient_id=True,
        dms_ignorer_date_admission=True,
    )
    worklist = copy.deepcopy(self.order_worklist)
    worklist.samples[0].orders.append(
        {
            "code": "D",
            "dilution": "",
            "analyseur_repasse": "",
            "action": "add",
            "priority": "A",
            "rerun": False,
        }
    )

    message = self.order(worklist=worklist)
    expected = """
    H|\\^&||||||||||P|1|20130715141912
    P|1|78910|036666666||NYME^ANO|NJF|20110513|F|||||PRESCRIBER|||1.70^cm|58^kg||||||||SER^SERVICE
    O|1|123456||^^^A^^^1:10|S||20130513114242||||N||||Serum||||||||||O
    O|2|123456||^^^B^^RR|S||20130513114242||||A||||Serum||||||||||O
    O|3|123456||^^^C|S||20130513114242||||C||||Serum||||||||||O
    O|4|123456||^^^D|S||20130513114242||||A||||Serum||||||||||O
    L|1
    """
    assert message == expected


def test_encode_order_with_serotheque(self):
    self.configure(dms_envoyer_no_serotheque=True)
    worklist = copy.deepcopy(self.order_worklist)
    worklist.samples[0].info_serotheque.codes = "SERO"
    worklist.samples[0].info_serotheque.portoir = "T0001"
    worklist.samples[0].info_serotheque.sample_id = "12346"
    worklist.samples[0].info_serotheque.destination = "PLATEAU"
    worklist.samples[0].info_serotheque.numero = "101"
    worklist.samples[0].info_serotheque.position = "1,1"
    worklist.samples[0].info_serotheque.etiquette = "T001 A,1"

    message = self.order(worklist=worklist)
    expected = """
    H|\\^&||||||||||P|1|20130715141912
    P|1|036666666|L036666666||NYME^ANO|NJF|20110513|F|||||PRESCRIBER|||1.70^cm|58^kg||||||20130511103030||SER^SERVICE
    O|1|123456||^^^A^^^1:10|S||20130513114242||||N||||Serum||||||||||O
    C|1|L|S018^123456^N|123456^^^^^PLATEAU^^^^^^^^^101|S
    O|2|123456||^^^B^^RR|S||20130513114242||||A||||Serum||||||||||O
    O|3|123456||^^^C|S||20130513114242||||C||||Serum||||||||||O
    L|1
    """
    assert message == expected


def test_encode_order_with_analyseur(self):
    worklist = copy.deepcopy(self.order_worklist)
    worklist.samples[0].orders.append(
        {
            "code": "D",
            "dilution": "",
            "action": "add",
            "priority": "A",
            "rerun": True,
            "analyseur_repasse": "VISTA1",
        }
    )

    message = self.order(worklist=worklist)
    expected = """
    H|\\^&||||||||||P|1|20130715141912
    P|1|036666666|L036666666||NYME^ANO|NJF|20110513|F|||||PRESCRIBER|||1.70^cm|58^kg||||||20130511103030||SER^SERVICE
    O|1|123456||^^^A^^^1:10|S||20130513114242||||N||||Serum||||||||||O
    O|2|123456||^^^B^^RR|S||20130513114242||||A||||Serum||||||||||O
    O|3|123456||^^^C|S||20130513114242||||C||||Serum||||||||||O
    O|4|123456||^^^D^^VISTA1|S||20130513114242||||A||||Serum||||||||||O
    L|1
    """
    assert message == expected


def test_decode_result(self):
    date = datetime.datetime(2009, 12, 22, 17, 30)
    worklist = self.decode(
        """
    H|\\^&|||DMS|||||||P|1|20150219122300
    P|1|AB123456|12345||TEST^ANONYME|ESSAI|19601201|F|W|120
    O|1|123456||^^^HCV^DMSTEST^RR^1:10|R||||||||||||||||||||F
    R|1|^^^HCV^10.0^HOSTTEST^DMSTEST^REAGENTLOT^^LOTCONTROL^^F| 0.275^NEG|mIU/mL||LL\\EXP||F||evh^DMS^20091222172700||20091222173000|I2000-1|0.45|imagefilename.ext
    C|1|I|Commentaire patient|I
    C|2|I|result^commentaire2|I
    L|1
    """
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "type": "patient",
                "sample_id": "123456",
                "results": [
                    {
                        "code": "HCV",
                        "comments": ["Commentaire patient"],
                        "date": date,
                        "flags": ["LL", "EXP"],
                        "instrument": "I2000-1",
                        "value": "0.275",
                        "dilution": "10.0",
                        "operator": "evh",
                        "lots": [{"lot": "REAGENTLOT", "type": "reagent"}],
                    },
                    {
                        "code": "HCV_I",
                        "comments": ["Commentaire patient"],
                        "date": date,
                        "flags": ["LL", "EXP"],
                        "instrument": "I2000-1",
                        "value": "NEG",
                        "dilution": "10.0",
                        "operator": "evh",
                        "lots": [{"lot": "REAGENTLOT", "type": "reagent"}],
                    },
                ],
            }
        ],
    }
    assert worklist == expected


def test_decode_control(self):
    self.configure(dms_qc_code_instrument="ADV2400DMS1;ADV2400DMS2")
    worklist = self.decode(
        """
    H|\\^&||182332||DMS|||||||P|1
    M|1|QC|K6202962^Cent_XPT_2^6202962^42312UD00^IRL06491415|aHCV|3.04|20150722122431|2.6150|0.2675|10.229|V|Nemo|2.99|0.05|1.672|1.588785046729|254||Index
    M|2|QC|PA001^ADV2400DMS1^45701^^CA12560033|LIP|32|20151005094321|0.0000|0.0000||V|Nemo|32.923|1.492|4.531||||
    M|3|QC|PA001^ADV2400DMS2^45701^^CA12560033|IP|18.12|20151005094321|0.0000|0.0000||V|Nemo|17.584|0.571|3.25||||
    L|1|N
    """
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "type": "control",
                "control_lot": "6202962",
                "sample_id": "6202962",
                "results": [
                    {
                        "code": "aHCV",
                        "value": "3.04",
                        "date": datetime.datetime(2015, 7, 22, 12, 24, 31),
                        "instrument": "Cent_XPT_2",
                        "lots": [{"lot": "42312UD00_254", "type": "reagent"}],
                    }
                ],
            },
            {
                "type": "control",
                "sample_id": "PA001",
                "control_lot": "PA001",
                "results": [
                    {
                        "code": "LIP",
                        "date": datetime.datetime(2015, 10, 5, 9, 43, 21),
                        "instrument": "ADV2400DMS1",
                        "value": "32",
                    },
                    {
                        "code": "IP",
                        "date": datetime.datetime(2015, 10, 5, 9, 43, 21),
                        "instrument": "ADV2400DMS2",
                        "value": "18.12",
                    },
                ],
            },
        ],
    }
    assert worklist == expected


def test_decode_control_cancel(self):
    self.configure(dms_qc_code_instrument="ADV2400DMS1;ADV2400DMS2")
    worklist = self.decode(
        """
    H|\\^&||182332||DMS|||||||P|1
    M|1|QC|PA001^ADV2400DMS1^45701^^CA12560033|LIP|32|20151005094321|0.0000|0.0000||V|Nemo|32.923|1.492|4.531|||D|
    L|1|N
    """
    )
    expected = {"context": "result", "samples": []}
    assert worklist == expected


def test_encode_enable_disable(self):
    worklist = driverng.Worklist(
        context="order",
        samples=[
            {
                "sample_id": "INSTRUMENT:ADV1800-1",
                "type": "patient",
                "orders": [
                    {"code": "CK", "action": "add"},
                    {"code": "NA", "action": "del"},
                ],
            },
            {
                "sample_id": "INSTRUMENT:ADV1800-2",
                "type": "patient",
                "orders": [{"code": "*", "action": "del"}],
            },
        ],
    )
    message = self.encode(worklist)
    assert (
        message
        == """
    H|\\^&||||||||||P|1|20130715141912
    M|1|ENABLE|ADV1800-1|CK
    M|2|DISABLE|ADV1800-1|NA
    M|3|DISABLE|ADV1800-2|*
    L|1"""
    )


def test_resultat_hemato(self):
    self.configure(analyseurs_hemato="Adv_2120_1;Adv_2120_2")
    date = datetime.datetime(2015, 11, 25, 9, 37, 44)
    worklist = self.decode(
        """
    H|\\^&||093757||DMS|||||||P|1
    P|1|01000080486|B5112401063||TEST^HEMATO||20051124|M||||||||0.000^|0.000^||||||20151125093757||||
    O|1|371711247075||^^^3^3^RR^|R||||||||||W^||||||||||F||
    R|1|^^^3^^^HGB^^^^1^F|9.90000^9.90000|N/A^N/A||CC||F||^Nemo^20151125093757||20151125093744|Adv_2120_2^|9.70000|371711247075_20151125093744.bmp
    O|2|371711247075||^^^908^908^RR^|R||||||||||W^||||||||||F||
    R|1|^^^908^^^%BL-AD^^^^1^F|1.30000^1.30000|N/A^N/A||||F||^Nemo^20151125093757||20151125093744|Adv_2120_2^|1.15000|371711247075_20151125093744.bmp
    O|3|371711247075||^^^5^5^RR^|R||||||||||W^||||||||||F||
    R|1|^^^5^^^VGM^^^^1^F|76.20000^76.20000|N/A^N/A||VF||F||^Nemo^20151125093757||20151125093744|Adv_2120_2^|76.15000|371711247075_20151125093744.bmp
    O|4|371711247075||^^^ABC^ABC^RR^|R||||||||||W^||||||||||F||
    R|1|^^^ABC^^^ABC^^^^1^F|76^76|N/A^N/A||VF||F||^Nemo^20151125093757||20151125093744|Centaur_1^|76.15000|371711247075_20151125093744.bmp
    L|1|N
    """
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "type": "patient",
                "sample_id": "371711247075",
                "results": [
                    {
                        "code": "3",
                        "comments": [],
                        "date": date,
                        "dilution": "",
                        "flags": ["CC"],
                        "instrument": "Adv_2120_2",
                        "value": "9.90000",
                    },
                    {
                        "code": "3_I",
                        "comments": [],
                        "date": date,
                        "dilution": "",
                        "flags": ["CC"],
                        "instrument": "Adv_2120_2",
                        "value": "9.90000",
                    },
                    {
                        "code": "908",
                        "comments": [],
                        "date": date,
                        "dilution": "",
                        "flags": [],
                        "instrument": "Adv_2120_2",
                        "value": "1.30000",
                    },
                    {
                        "code": "908_I",
                        "comments": [],
                        "date": date,
                        "dilution": "",
                        "flags": [],
                        "instrument": "Adv_2120_2",
                        "value": "1.30000",
                    },
                    {
                        "code": "5",
                        "comments": [],
                        "date": date,
                        "dilution": "",
                        "flags": ["VF"],
                        "instrument": "Adv_2120_2",
                        "value": "76.20000",
                    },
                    {
                        "code": "5_I",
                        "comments": [],
                        "date": date,
                        "dilution": "",
                        "flags": ["VF"],
                        "instrument": "Adv_2120_2",
                        "value": "76.20000",
                    },
                    {
                        "code": "ABC",
                        "comments": [],
                        "date": date,
                        "dilution": "",
                        "flags": ["VF"],
                        "instrument": "Centaur_1",
                        "value": "76",
                    },
                    {
                        "code": "ABC_I",
                        "comments": [],
                        "date": date,
                        "dilution": "",
                        "flags": ["VF"],
                        "instrument": "Centaur_1",
                        "value": "76",
                    },
                    {"code": "ALARME", "comments": ["CC", "VF"], "value": ""},
                ],
            }
        ],
    }
    assert worklist == expected


def test_decode_result_liaison(self):
    date = datetime.datetime(2017, 2, 1, 18, 17, 31)
    self.configure(dms_dilution_analyseur="LiaisonLxL;LIAISONXL")
    worklist = self.decode(
        """
    H|\\^&||182614||DMS|||||||P|1
    P|1|99010004|1701310028||Z_LABO^Informatique||20000330|F||||||||0.000^|0.000^||||||20170201182614||||
    O|1|9901000401||^^^VCAGL^VCAGL^RR^20|R||20170201144600||||||||S^||||||||||F||
    R|1|^^^VCAGL^^^VGA-G^30073^2300^^1^F|580^POS^^|U/mL^||N||F||^Nemo^20170201182614||20170201181731|LIAISONXL^lxllas0951|290
    L|1|N
    """
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "type": "patient",
                "sample_id": "9901000401",
                "results": [
                    {
                        "code": "VCAGL",
                        "comments": [],
                        "date": date,
                        "dilution": "20",
                        "flags": ["N"],
                        "instrument": "LIAISONXL",
                        "lots": [{"lot": "30073_2300", "type": "reagent"}],
                        "value": "580",
                    },
                    {
                        "code": "VCAGL_I",
                        "comments": [],
                        "date": date,
                        "dilution": "20",
                        "flags": ["N"],
                        "instrument": "LIAISONXL",
                        "lots": [{"lot": "30073_2300", "type": "reagent"}],
                        "value": "POS",
                    },
                ],
            }
        ],
    }
    assert worklist == expected


def test_decode_control_lxl(self):
    self.configure(lxl_ref_qc_tronquee=True)
    worklist = self.decode(
        """
    H|\\^&||102353||DMS|||||||P|1
    M|1|QC|#5410350^lxl01559^#5410350^30070^lxl01559|VCAGI2|11|20160314103932|0.0000|0.0000||N|Nemo|0|0|||12887|<|
    L|1|N
    """
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "type": "control",
                "control_lot": "5410350",
                "sample_id": "5410350",
                "results": [
                    {
                        "code": "VCAGI2",
                        "value": "11",
                        "date": datetime.datetime(2016, 3, 14, 10, 39, 32),
                        "instrument": "lxl01559",
                        "lots": [{"lot": "30070_12887", "type": "reagent"}],
                    }
                ],
            }
        ],
    }
    assert worklist == expected


def test_decode_control_acltop(self):
    self.configure(dms_qc_acltop="ACLTOP750")
    worklist = self.decode(
        """
    H|\\^&||122312||DMS|||||||P|1
    M|1|QC|CQ DDi L2^ACLTOP750^B26607^B27139^11|DDIM|2211^|20170504122056|0.0000|0.0000||N|Nemo|1934.267|116.913|6.044|||N|ng/mL
    L|1|N
    """
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "type": "control",
                "control_lot": "CQ DDi L2-B26607",
                "sample_id": "CQ DDi L2-B26607",
                "results": [
                    {
                        "code": "DDIM",
                        "value": "2211",
                        "date": datetime.datetime(2017, 5, 4, 12, 20, 56),
                        "instrument": "ACLTOP750",
                        "lots": [{"lot": "B27139", "type": "reagent"}],
                    }
                ],
            }
        ],
    }
    assert worklist == expected


def test_decode_result_with_rack(self):
    worklist = self.decode(
        """
    H|\\^&||130827||DMS|||||||P|1
    P|1|49920175|110322168||ESSAI^Test||19500101|M|||||HVERS|||0.000^|0.000^||||||20180516130827||||
    O|1|49920175||^^^NA^NA^^|R||20180516130500||||||||S^^||||||||||F||
    R|1|^^^NA^^^NA^^^^1^F|No Result^No Result^^|N/A^N/A||UNABLE_TO_DO||F||^Nemo^20180516130827||20180516130821|ATELL_2^|0||^004682^A11^
    L|1|N
    """
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "type": "patient",
                "sample_id": "49920175",
                "rack": "004682-A11",
                "results": [
                    {
                        "code": "NA",
                        "comments": [],
                        "date": datetime.datetime(2018, 5, 16, 13, 8, 21),
                        "flags": ["UNABLE_TO_DO"],
                        "instrument": "ATELL_2",
                        "value": "No Result",
                        "dilution": "",
                    },
                    {
                        "code": "NA_I",
                        "comments": [],
                        "date": datetime.datetime(2018, 5, 16, 13, 8, 21),
                        "flags": ["UNABLE_TO_DO"],
                        "instrument": "ATELL_2",
                        "value": "No Result",
                        "dilution": "",
                    },
                ],
            }
        ],
    }
    assert worklist == expected


def test_decode_result_with_ignore_rack(self):
    self.configure(dms_ignorer_rack=True)
    worklist = self.decode(
        """
    H|\\^&||130827||DMS|||||||P|1
    P|1|49920175|110322168||ESSAI^Test||19500101|M|||||HVERS|||0.000^|0.000^||||||20180516130827||||
    O|1|49920175||^^^NA^NA^^|R||20180516130500||||||||S^^||||||||||F||
    R|1|^^^NA^^^NA^^^^1^F|No Result^No Result^^|N/A^N/A||UNABLE_TO_DO||F||^Nemo^20180516130827||20180516130821|ATELL_2^|0||^004682^A11^
    L|1|N
    """
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "type": "patient",
                "sample_id": "49920175",
                "results": [
                    {
                        "code": "NA",
                        "comments": [],
                        "date": datetime.datetime(2018, 5, 16, 13, 8, 21),
                        "flags": ["UNABLE_TO_DO"],
                        "instrument": "ATELL_2",
                        "value": "No Result",
                        "dilution": "",
                    },
                    {
                        "code": "NA_I",
                        "comments": [],
                        "date": datetime.datetime(2018, 5, 16, 13, 8, 21),
                        "flags": ["UNABLE_TO_DO"],
                        "instrument": "ATELL_2",
                        "value": "No Result",
                        "dilution": "",
                    },
                ],
            }
        ],
    }
    assert worklist == expected


def test_decode_result_with_calibration(self):
    worklist = self.decode(
        """
    H|\\^&||174637||DMS|||||||P|1
    P|1|890124109857|1479047||ESSAI MCA^DMS||19870305|F||||||||0.000^|0.000^||||||20181024174637||||
    O|1|890124109857||^^^658^658^RR^|R||20181024170000||||||||^^||||||||||F||
    R|1|^^^658^UNDILUTED;^^BHCG^87414UI00^02821^^1^F|25.270^Positive^^|IU/L^||||F||FAS;FAS^Nemo^20181024174637||20181024174636|01-ALi-02^Ai01645|0||^LAS^1^
    M|1|CAL|658|01-ALi-02||20180913191750|
    M|2|INV|SR|658-1|87414UI00|02821|20190416
    R|2|^^^658^UNDILUTED;^^BHCG^87414UI00^02821^^1^F|23.890^ <Blank> ^^|IU/L^||||P||FAS;FAS^Nemo^20181024174637||20181024172632|01-ALi-02^Ai01645|0||^LAS^1^
    M|3|CAL|658|01-ALi-02||20180913191750|
    M|4|INV|SR|658-1|87414UI00|02821|20190416
    L|1|N
    """
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "type": "patient",
                "sample_id": "890124109857",
                "rack": "LAS-1",
                "results": [
                    {
                        "code": "658",
                        "comments": [],
                        "date": datetime.datetime(2018, 10, 24, 17, 46, 36),
                        "dilution": "UNDILUTED;",
                        "flags": [],
                        "instrument": "01-ALi-02",
                        "lots": [{"lot": "87414UI00_02821", "type": "reagent"}],
                        "operator": "FAS;FAS",
                        "value": "25.270",
                    },
                    {
                        "code": "658_I",
                        "comments": [],
                        "date": datetime.datetime(2018, 10, 24, 17, 46, 36),
                        "dilution": "UNDILUTED;",
                        "flags": [],
                        "instrument": "01-ALi-02",
                        "lots": [{"lot": "87414UI00_02821", "type": "reagent"}],
                        "operator": "FAS;FAS",
                        "value": "Positive",
                    },
                ],
            }
        ],
    }
    assert worklist == expected


def test_decode_resultats_with_reagents(self):
    worklist = self.decode(
        """
    H|\\^&||112358||DMS|||||||P|1
    P|1|890124109809|1479047||ESSAI MCA^DMS||19870305|F||||||||0.000^|0.000^||||||20181024112358||||
    O|1|890124109809||^^^1072^1072^^|R||20181024110100||||||||^^||||||||||F||
    R|1|^^^1072^^^ICT^00001^0010^^1^F|0.000^ <Blank> ^^|Index^||||F||FAS;FAS^Nemo^20181024112358||20181024112355|01-ALC-01^AC01305|0||^LAS^1^
    M|1|INV|SR|HILS0-1|00001|0010|20190603
    O|2|890124109809||^^^1073^1073^^|R||20181024110100||||||||^^||||||||||F||
    R|1|^^^1073^^^LIP^00001^0010^^1^F|-7^ <Blank> ^^|Index^||||F||FAS;FAS^Nemo^20181024112358||20181024112355|01-ALC-01^AC01305|0||^LAS^1^
    M|2|INV|SR|HILS0-1|00001|0010|20190603
    L|1|N
    """
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "type": "patient",
                "sample_id": "890124109809",
                "rack": "LAS-1",
                "results": [
                    {
                        "code": "1072",
                        "comments": [],
                        "date": datetime.datetime(2018, 10, 24, 11, 23, 55),
                        "dilution": "",
                        "flags": [],
                        "instrument": "01-ALC-01",
                        "lots": [{"lot": "00001_0010", "type": "reagent"}],
                        "operator": "FAS;FAS",
                        "value": "0.000",
                    },
                    {
                        "code": "1072_I",
                        "comments": [],
                        "date": datetime.datetime(2018, 10, 24, 11, 23, 55),
                        "dilution": "",
                        "flags": [],
                        "instrument": "01-ALC-01",
                        "lots": [{"lot": "00001_0010", "type": "reagent"}],
                        "operator": "FAS;FAS",
                        "value": "<Blank>",
                    },
                    {
                        "code": "1073",
                        "comments": [],
                        "date": datetime.datetime(2018, 10, 24, 11, 23, 55),
                        "dilution": "",
                        "flags": [],
                        "instrument": "01-ALC-01",
                        "lots": [{"lot": "00001_0010", "type": "reagent"}],
                        "operator": "FAS;FAS",
                        "value": "-7",
                    },
                    {
                        "code": "1073_I",
                        "comments": [],
                        "date": datetime.datetime(2018, 10, 24, 11, 23, 55),
                        "dilution": "",
                        "flags": [],
                        "instrument": "01-ALC-01",
                        "lots": [{"lot": "00001_0010", "type": "reagent"}],
                        "operator": "FAS;FAS",
                        "value": "<Blank>",
                    },
                ],
            }
        ],
    }
    assert worklist == expected


def test_decode_calibration(self):
    worklist = self.decode(
        """
    H|\\^&||103206||DMS|||||||P|1
    M|1|CALIBRATOR|52930UQ051|1134|01-ALC-01^AC01305|19.9664^mV^|F|52930UQ05||20181024102959|Admin;Admin
    M|2|INV|SR|ICTD5-1|28639UN18|01706|20190529
    M|3|CALIBRATOR|52930UQ051|1133|01-ALC-01^AC01305|-28.2979^mV^|F|52930UQ05||20181024102959|Admin;Admin
    M|4|INV|SR|ICTD5-1|28639UN18|01706|20190529
    M|5|CALIBRATOR|52930UQ051|1134|01-ALC-01^AC01305|19.9435^mV^|F|52930UQ05||20181024103023|Admin;Admin
    M|6|INV|SR|ICTD5-1|28639UN18|01706|20190529
    L|1|N
    """
    )
    expected = {"context": "result", "samples": []}
    assert worklist == expected


def test_decode_result_replace_decimal(self):
    date = datetime.datetime(2020, 2, 13, 12, 42, 38)
    worklist = self.decode(
        """
    H|\\^&||112358||DMS|||||||P|1
    P|1|890124109809|1479047||ESSAI MCA^DMS||19870305|F||||||||0.000^|0.000^||||||20181024112358||||
    O|1|L080435043PR||^^^TOP^TOP^^|R||20200213082600||||||||^^||||||||||F||
    R|1|^^^TOP^^^ALATOP^^^^1^F|1,00^1,00^^|^||N||F||^Nemo^20200213124607||20200213124238|14-I2K-01^IMMULITE 2|1||^^^
    L|1|N
    """
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "type": "patient",
                "sample_id": "L080435043PR",
                "results": [
                    {
                        "code": "TOP",
                        "comments": [],
                        "date": date,
                        "dilution": "",
                        "flags": ["N"],
                        "instrument": "14-I2K-01",
                        "value": "1.00",
                    },
                    {
                        "code": "TOP_I",
                        "comments": [],
                        "date": date,
                        "dilution": "",
                        "flags": ["N"],
                        "instrument": "14-I2K-01",
                        "value": "1.00",
                    },
                ],
            }
        ],
    }
    assert worklist == expected


def test_decode_resultats_with_instrument_limit(self):
    for instrument_length, instrument_name in (
        (False, "01-ALC-01"),  # on ne tronque pas
        (0, "01-ALC-01"),  # on ne tronque pas
        (9, "01-ALC-01"),  # on tronque à 9 chars: rien ne change
        (7, "01-ALC-"),  # on tronque à 7 chars: 2 chars en moins
        (-3, "01-ALC-01"),  # mauvaise valeur: on ne tronque pas
    ):
        self.configure(dms_longueur_analyseur=instrument_length)
        worklist = self.decode(
            """
        H|\\^&||112358||DMS|||||||P|1
        P|1|890124109809|1479047||ESSAI MCA^DMS||19870305|F||||||||0.000^|0.000^||||||20181024112358||||
        O|1|890124109809||^^^1072^1072^^|R||20181024110100||||||||^^||||||||||F||
        R|1|^^^1072^^^ICT^00001^0010^^1^F|0.000^ <Blank> ^^|Index^||||F||FAS;FAS^Nemo^20181024112358||20181024112355|01-ALC-01^AC01305|0||^LAS^1^
        M|1|INV|SR|HILS0-1|00001|0010|20190603
        L|1|N
        """
        )
        expected = {
            "context": "result",
            "samples": [
                {
                    "type": "patient",
                    "sample_id": "890124109809",
                    "rack": "LAS-1",
                    "results": [
                        {
                            "code": "1072",
                            "comments": [],
                            "date": datetime.datetime(2018, 10, 24, 11, 23, 55),
                            "dilution": "",
                            "flags": [],
                            "instrument": instrument_name,
                            "lots": [{"lot": "00001_0010", "type": "reagent"}],
                            "operator": "FAS;FAS",
                            "value": "0.000",
                        },
                        {
                            "code": "1072_I",
                            "comments": [],
                            "date": datetime.datetime(2018, 10, 24, 11, 23, 55),
                            "dilution": "",
                            "flags": [],
                            "instrument": instrument_name,
                            "lots": [{"lot": "00001_0010", "type": "reagent"}],
                            "operator": "FAS;FAS",
                            "value": "<Blank>",
                        },
                    ],
                }
            ],
        }
        assert worklist == expected, (
            "instrument_length={} ; "
            "instrument_name={}".format(instrument_length, instrument_name)
        )


def test_decode_result_whith_status(self):
    self.configure(dms_status_integre="F;P")
    worklist = self.decode(
        """
    H|\\^&||181952||DMS|||||||P|1
    P|1|REP2|D5091501346||Z_labo^TOX|MI|19940505|F||||||||0.000^|0.000^||||||20190829181952||||
    O|1|REP2||^^^1839^1839^RR^|R||20190828170831||||||||^^||||||||||F||
    R|1|^^^1839^STANDARD;^^MALB^64154UN18^^^1^F|142^142^^|mg/L^mg/L||||F||Admin;Admin^Nemo^20190829181952||20190828170738|07-Ac-01^AC01955|141.733||^N2316^2^
    M|1|CAL|1839|07-Ac-01||20190823110713|20190920110713
    M|2|INV|SR|UALB0-1|64154UN18||20191115
    R|2|^^^1839^STANDARD;^^MALB^64154UN18^^^1^F|142^142^^|mg/L^mg/L||||P||Admin;Admin^Nemo^20190829181952||20190828170738|07-Ac-01^AC01955|141.733||^N2316^2^
    M|3|CAL|1839|07-Ac-01||20190823110713|20190920110713
    M|4|INV|SR|UALB0-1|64154UN18||20191115
    L|1|N
    """
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "type": "patient",
                "sample_id": "REP2",
                "rack": "N2316-2",
                "results": [
                    {
                        "code": "1839",
                        "value": "142",
                        "date": datetime.datetime(2019, 8, 28, 17, 7, 38),
                        "instrument": "07-Ac-01",
                        "dilution": "STANDARD;",
                        "operator": "Admin;Admin",
                        "comments": [],
                        "flags": [],
                        "lots": [{"lot": "64154UN18", "type": "reagent"}],
                    },
                    {
                        "code": "1839_I",
                        "value": "142",
                        "date": datetime.datetime(2019, 8, 28, 17, 7, 38),
                        "instrument": "07-Ac-01",
                        "dilution": "STANDARD;",
                        "operator": "Admin;Admin",
                        "comments": [],
                        "flags": [],
                        "lots": [{"lot": "64154UN18", "type": "reagent"}],
                    },
                    {
                        "code": "1839",
                        "value": "142",
                        "date": datetime.datetime(2019, 8, 28, 17, 7, 38),
                        "instrument": "07-Ac-01",
                        "dilution": "STANDARD;",
                        "operator": "Admin;Admin",
                        "comments": [],
                        "flags": [],
                        "lots": [{"lot": "64154UN18", "type": "reagent"}],
                    },
                    {
                        "code": "1839_I",
                        "value": "142",
                        "date": datetime.datetime(2019, 8, 28, 17, 7, 38),
                        "instrument": "07-Ac-01",
                        "dilution": "STANDARD;",
                        "operator": "Admin;Admin",
                        "comments": [],
                        "flags": [],
                        "lots": [{"lot": "64154UN18", "type": "reagent"}],
                    },
                ],
            }
        ],
    }
    assert worklist == expected
