# -*- coding: utf8 -*-
from __future__ import unicode_literals

import datetime
import driverng

parser_class = "facs_workflow_manager"


def test_encode_order(self):
    worklist = driverng.Worklist(
        context="order",
        samples=[
            {
                "sample_id": "SID4090",
                "type": "patient",
                "orders": [{"code": "6CTBNK", "action": "add", "dilution": "1"}],
                "collection_date": datetime.datetime(2013, 5, 13, 12, 42, 42),
                "request_date": datetime.datetime(2013, 5, 13, 11, 42, 42),
                "physicians": [
                    {
                        "code": "PRESCRIBER",
                        "last_name": "Prescriber",
                        "type": "prescriber",
                    }
                ],
                "last_name": "Doe",
                "first_name": "John",
                "birthdate": datetime.datetime(1978, 2, 11, 12, 15),
                "sex": "M",
                "patient_id": "PID12345",
                "priority": "R",
                "height": "176",
                "weight": "76",
                "specimen": "B",
                "comments": [
                    {"type": "patient", "text": "Patient commentaire"},
                    {"type": "sample", "text": "Sample commentaire"},
                ],
                "request_comment_obr": "Request comment",
                "request_comment_patient": "Patient comment",
            }
        ],
    )
    worklist.samples[0].results = [
        {
            "code": "CD3C",
            "value": "440",
            "date": datetime.datetime(2013, 5, 13, 13, 42, 42),
            "comments": ["These are comments"],
        }
    ]
    message = self.order(worklist=worklist, context={"control_id": "23"})
    expected = """
    MSH|^~\\&|||||||ORM^O01|23|P|2.5
    PID|1||PID12345||Doe^John||19780211|M
    PV1|1
    ORC|NW|SID4090
    OBR|1|SID4090||6CTBNK||20130513114242|20130513124242||||||176^76||B|PRESCRIBER
    NTE|1|L|Sample commentaire
    NTE|2|L|Request comment
    TCD|1|||1
    OBX|1||CD3C|6CTBNK|440|||||||||20130513134242
    NTE|1|L|These are comments
    """
    assert message == expected


def test_encode_order_frn(self):
    worklist = driverng.Worklist(
        context="order",
        samples=[
            {
                "sample_id": "SAMPLE-ID-XXXX",
                "type": "patient",
                "orders": [{"code": "TBNK", "action": "add"}],
                "collection_date": datetime.datetime(2022, 6, 7, 8, 5),
                "request_date": datetime.datetime(2022, 6, 7, 8, 5),
                "physicians": [
                    {
                        "code": "Dr.Zhireça",
                        "last_name": "Dr.Zhireça",
                        "type": "prescriber",
                    }
                ],
                "last_name": "Hill",
                "first_name": "Bill",
                "birthdate": datetime.datetime(1974, 6, 30),
                "sex": "M",
                "patient_id": "PID-5645",
                "alternate_patient_id": "936996576",
                "priority": "R",
                "height": "198",
                "weight": "100",
                "specimen": "B",
            }
        ],
    )
    message = self.order(
        worklist=worklist,
        context={"control_id": "ffb06fca-938a-4b78-97d8-6f74221d0077"},
    )
    expected = """
    MSH|^~\\&|||||||ORM^O01|ffb06fca-938a-4b78-97d8-6f74221d0077|P|2.5
    PID|1||PID-5645||Hill^Bill||19740630|M|||||||||||936996576
    PV1|1
    ORC|NW|SAMPLE-ID-XXXX
    OBR|1|SAMPLE-ID-XXXX||TBNK||20220607080500|20220607080500||||||198^100||B|Dr.Zhireça
    """
    assert message == expected


def test_decode_result(self):
    worklist = self.decode(
        """
    MSH|^~\\&|FWM_Version_1.0||LIS||20190903133524||ORU^R01|1021^563|P|2.5.1|||AL|NE
    PID|1||PID12345||Doe^John||19740921|M|||||||||||001499991
    PV1|1
    ORC|RE|SID15
    OBR|1|SID4090||6CTBNK||20201119074500|20201119070000|20201123113513|||||180^80|||||LYRIC-DESK1^123456||Worklist_004_6 Color TBNK_SID4090_Lab Report.pdf^Worklist_004_6 Color TBNK_SID4090_Physician Report.pdf^Worklist_004_6 Color TBNK_SID4090_Supplemental Report.pdf
    OBX|1||CD45C|6CTBNK_TC|1500.00^HIGHT|cells/µl||||||||20201208174239||||Lyric-1^
    OBX|2||CD3P|6CTBNK_TC|40^LOW|%||||||||20201208174239||||^54321
    NTE|1|L|These are comments
    """
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "results": [
                    {
                        "code": "LABORATORY_CRPDF",
                        "value": "Worklist_004_6 Color TBNK_SID4090_Lab Report.pdf",
                    },
                    {
                        "code": "PHYSICIAN_CRPDF",
                        "value": "Worklist_004_6 Color TBNK_SID4090_Physician Report.pdf",
                    },
                    {
                        "code": "SUPPLEMENTAL_CRPDF",
                        "value": "Worklist_004_6 Color TBNK_SID4090_Supplemental Report.pdf",
                    },
                    {
                        "code": "CD45C",
                        "date": datetime.datetime(2020, 12, 8, 17, 42, 39),
                        "flags": ["HIGHT"],
                        "instrument": "Lyric-1",
                        "operator": "",
                        "unit": "cells/µl",
                        "value": "1500.00",
                    },
                    {
                        "code": "CD3P",
                        "comments": ["These are comments"],
                        "date": datetime.datetime(2020, 12, 8, 17, 42, 39),
                        "flags": ["LOW"],
                        "instrument": "54321",
                        "operator": "",
                        "unit": "%",
                        "value": "40",
                    },
                ],
                "sample_id": "SID4090",
                "type": "patient",
            }
        ],
    }
    assert worklist == expected


def test_decode_result_frn(self):
    worklist = self.decode(
        """
    MSH|^~\\&|FWM_Version_1.0||LIS||20220607160143||ORU^R01|127-113|P|2.5.1|||AL|NE|NULL|8859/1|
    PID|1||PID-5645||Hill^Bill||19740630|Male|
    ORC|RE|SAMPLE-ID-XXXX
    OBR|1|SAMPLE-ID-XXXX||CD45C~CD3P~CD3C~CD19P~CD19C~CD16_56P~CD16_56C~CD3_16_56P~CD3_16_56C||20220607080500|20220607080500|20220607180029|^||||||B|||VM-LYRIC^12345||Worklist_014_3-16+56-45-19_SAMPLE-ID-XXXX_Lab Report.pdf^Worklist_014_3-16+56-45-19_SAMPLE-ID-XXXX_Physician Report.pdf^Worklist_014_3-16+56-45-19_SAMPLE-ID-XXXX_Supplemental Report.pdf|||||||||||||||
    OBX|1||CD45C|TBNK|500.00^|cells/µl|||||R|||20220607180029||||VM-LYRIC^12345|
    OBX|2||CD3P|TBNK|44.25^|%|55.25 - 75.19||||R|||20220607180029||||VM-LYRIC^12345|
    OBX|3||CD3C|TBNK|221.00^|cells/µl|500.00 - 1612.00||||R|||20220607180029||||VM-LYRIC^12345|
    OBX|4||CD19P|TBNK|10.82^|%|||||R|||20220607180029||||VM-LYRIC^12345|
    OBX|5||CD19C|TBNK|54.00^|cells/µl|||||R|||20220607180029||||VM-LYRIC^12345|
    OBX|6||CD16_56P|TBNK|43.01^|%|||||R|||20220607180029||||VM-LYRIC^12345|
    OBX|7||CD16_56C|TBNK|215.00^|cells/µl|||||R|||20220607180029||||VM-LYRIC^12345|
    OBX|8||CD3_16_56P|TBNK|2.84^|%|||||R|||20220607180029||||VM-LYRIC^12345|
    OBX|9||CD3_16_56C|TBNK|14.00^|cells/µl|||||R|||20220607180029||||VM-LYRIC^12345|
    """
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "results": [
                    {
                        "code": "LABORATORY_CRPDF",
                        "value": "Worklist_014_3-16+56-45-19_SAMPLE-ID-XXXX_Lab Report.pdf",
                    },
                    {
                        "code": "PHYSICIAN_CRPDF",
                        "value": "Worklist_014_3-16+56-45-19_SAMPLE-ID-XXXX_Physician Report.pdf",
                    },
                    {
                        "code": "SUPPLEMENTAL_CRPDF",
                        "value": "Worklist_014_3-16+56-45-19_SAMPLE-ID-XXXX_Supplemental Report.pdf",
                    },
                    {
                        "code": "CD45C",
                        "date": datetime.datetime(2022, 6, 7, 18, 0, 29),
                        "instrument": "12345",
                        "operator": "",
                        "unit": "cells/µl",
                        "value": "500.00",
                    },
                    {
                        "code": "CD3P",
                        "date": datetime.datetime(2022, 6, 7, 18, 0, 29),
                        "instrument": "12345",
                        "operator": "",
                        "unit": "%",
                        "value": "44.25",
                    },
                    {
                        "code": "CD3C",
                        "date": datetime.datetime(2022, 6, 7, 18, 0, 29),
                        "instrument": "12345",
                        "operator": "",
                        "unit": "cells/µl",
                        "value": "221.00",
                    },
                    {
                        "code": "CD19P",
                        "date": datetime.datetime(2022, 6, 7, 18, 0, 29),
                        "instrument": "12345",
                        "operator": "",
                        "unit": "%",
                        "value": "10.82",
                    },
                    {
                        "code": "CD19C",
                        "date": datetime.datetime(2022, 6, 7, 18, 0, 29),
                        "instrument": "12345",
                        "operator": "",
                        "unit": "cells/µl",
                        "value": "54.00",
                    },
                    {
                        "code": "CD16_56P",
                        "date": datetime.datetime(2022, 6, 7, 18, 0, 29),
                        "instrument": "12345",
                        "operator": "",
                        "unit": "%",
                        "value": "43.01",
                    },
                    {
                        "code": "CD16_56C",
                        "date": datetime.datetime(2022, 6, 7, 18, 0, 29),
                        "instrument": "12345",
                        "operator": "",
                        "unit": "cells/µl",
                        "value": "215.00",
                    },
                    {
                        "code": "CD3_16_56P",
                        "date": datetime.datetime(2022, 6, 7, 18, 0, 29),
                        "instrument": "12345",
                        "operator": "",
                        "unit": "%",
                        "value": "2.84",
                    },
                    {
                        "code": "CD3_16_56C",
                        "date": datetime.datetime(2022, 6, 7, 18, 0, 29),
                        "instrument": "12345",
                        "operator": "",
                        "unit": "cells/µl",
                        "value": "14.00",
                    },
                ],
                "sample_id": "SAMPLE-ID-XXXX",
                "type": "patient",
            }
        ],
    }
    assert worklist == expected
