# -*- coding: utf8 -*-
from __future__ import unicode_literals

import datetime

parser_class = "kpcr_astm"


def test_encode_order(self):
    message = self.order()
    expected = """
    H|\\^&
    P|1
    O|1|123456||^^^A\\^^^B|||||||||||||||||||||
    L|1
    """
    assert message == expected


def test_decode_result(self):
    date = datetime.datetime(2016, 5, 16, 11, 52, 24)
    worklist = self.decode(
        """
    H|\\^&|||VERSANT_kPCR^1.0|||||ACME_LIS||P|1|20160516114853
    P|1||||^^^^|||U||^^^|||^^^^^|||||||||||||||||||||
    O|1|0888006003604^B^4||^^^IC^^^^\\^^^IC-Ct^^^^\\^^^HIV^^^^\\^^^HIVCt^^^^|R|18991230000000|||||||||OT||||||||||F
    R|1|^^^IC^^^^|14.56|||N||F||RSmith||20160516115224|VersantkPCR1
    C|1|I|Notes=Above Range|G
    R|2|^^^IC-Ct^^^^|20.50|||N||F||RSmith||20160516115224|VersantkPCR1
    R|3|^^^HIV^^^^|Negatif|copies/mL||N||F||RSmith||20160516115224|VersantkPCR1
    R|4|^^^HIV-Ct^^^^|Invalid Result|||N||F||RSmith||20160516115224|VersantkPCR1
    L|1|N
    """
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "type": "patient",
                "sample_id": "0888006003604",
                "results": [
                    {
                        "code": "IC",
                        "value": "14.56",
                        "flags": ["N"],
                        "comments": ["Notes=Above Range"],
                        "date": date,
                        "instrument": "VersantkPCR1",
                    },
                    {
                        "code": "IC-Ct",
                        "value": "20.50",
                        "flags": ["N"],
                        "comments": [],
                        "date": date,
                        "instrument": "VersantkPCR1",
                    },
                    {
                        "code": "HIV",
                        "value": "Negatif",
                        "flags": ["N"],
                        "comments": [],
                        "date": date,
                        "instrument": "VersantkPCR1",
                    },
                    {
                        "code": "HIV-Ct",
                        "value": "Invalid Result",
                        "flags": ["N"],
                        "comments": [],
                        "date": date,
                        "instrument": "VersantkPCR1",
                    },
                ],
            }
        ],
    }
    assert worklist == expected
